<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /VisitorsController.php
* @date         20/12/2011
*/

class VisitorsController extends PluginController {
    
    const PLUGIN_NAME = 'visitors'; 
    
    public function __construct() {
        if (defined('CMS_BACKEND')) {
            define('VIEWS', PLUGINS_ROOT . DS . self::PLUGIN_NAME . DS . 'views' . DS);
            
            $this->setLayout('backend');
            $this->assignToLayout('sidebar', new View(VIEWS . 'sidebar'));
        }
        else {
            exit();
        }
    }
    
    public function browsers() {
        $this->display('visitors/views/browsers', array(
            'browser_families' => VisitorsBrowser::findBrowserFamily(20),
            'browsers' => VisitorsBrowser::findBrowser(20),
            'operation_system_families' => VisitorsBrowser::findOSFamily(20),
            'operation_systems' => VisitorsBrowser::findOS(20)
        ));
    }
    
    public function countries() {
        $this->display('visitors/views/countries', array(
            'countries' => VisitorsCountry::findTop(50)
        ));
    }   
    
    public function dashboard() {
        $this->display('visitors/views/dashboard', array(
            'keywords' => VisitorsKeywords::findPopular(10),
            'pages' => VisitorsPage::findTop(10),
            'referring_domains' => VisitorsReferringDomain::findTop(5),
            'browser_families' => VisitorsBrowser::findBrowserFamily(5),
            'operation_system_families' => VisitorsBrowser::findOSFamily(5),
            'robots' => VisitorsBrowser::findRobot(5),
            'countries' => VisitorsCountry::findTop(5)
        ));
    }
    
    public function documentation() {
        $locale = strtolower(i18n::getLocale());
        $local_doc = $locale . '-documentation';
        $path = PLUGINS_ROOT . '/visitors/views/documentation/';

        if (!file_exists( $path . $local_doc . '.php' )) {
            $local_doc = 'en-documentation';
        }

        $this->display('visitors/views/documentation/' . $local_doc );
    }
    
    public function index() {
        $this->dashboard();
    }
    
    public function keywords() {
        $this->display('visitors/views/keywords', array(
            'keywords' => VisitorsKeywords::findPopular(50)
        ));
    }
    
    public function pages() {
        $this->display('visitors/views/pages', array(
            'pages' => VisitorsPage::findTop(50)
        ));
    }
    
    public function referringDomains() {
        $this->display('visitors/views/referring_domains', array(
            'referring_domains' => VisitorsReferringDomain::findTop(50)
        ));
    }
    
    public function robots() {
        $this->display('visitors/views/robots', array(
            'robots' => VisitorsBrowser::findRobot(50)
        ));
    }
    
    public function settings() {
        if (isset($_POST['save']) && $_POST['save'] == __('Save Settings')) {
            Plugin::setAllSettings($_POST['setting'], self::PLUGIN_NAME);
            Flash::setNow('success', __('Settings have been saved!'));
        }
        
        $this->display('visitors/views/settings', array(
            'settings' => Plugin::getAllSettings(self::PLUGIN_NAME),
            'layouts'  => Layout::findAll(array())
        ));
    }
    
    public function visitors() {
        $this->display('visitors/views/visitors', array(
            'visitors' => VisitorsVisitor::findRecentHuman(50)
        ));
    }
}