<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /models/VisitorsBrowser.php
* @date         27/12/2011
*/

class VisitorsBrowser extends Record
{
    
    public static function find($args = null) {
        // Collect attributes...
        $where    = isset($args['where']) ? trim($args['where']) : '';
        $group_by = isset($args['group']) ? trim($args['group']) : '';
        $order_by = isset($args['order']) ? trim($args['order']) : '';
        $offset   = isset($args['offset']) ? (int) $args['offset'] : 0;
        $limit    = isset($args['limit']) ? (int) $args['limit'] : 0;

        // Prepare query parts
        $where_string = empty($where) ? '' : "WHERE $where";
        $group_by_string = empty($group_by) ? '' : "GROUP BY $group_by";
        $order_by_string = empty($order_by) ? '' : "ORDER BY $order_by";
        $limit_string = $limit > 0 ? "LIMIT $limit" : '';
        $offset_string = $offset > 0 ? "OFFSET $offset" : '';

        $tablename = self::tableNameFromClassName('VisitorsRequest');
        $user_agent_table = self::tableNameFromClassName('VisitorsUserAgent');

        // Prepare SQL
        $sql = "SELECT
                    ua.*,
                    COUNT(log.id) AS hit_count,
                    COUNT(DISTINCT log.remote_addr, http_user_agent) AS visitor_count,
                        COUNT(DISTINCT remote_addr, http_user_agent)
                        /
                        (SELECT COUNT(DISTINCT remote_addr, http_user_agent) FROM $tablename AS log LEFT JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent $where_string)
                    * 100 AS percentage,
                        COUNT(log.id)
                        /
                        (SELECT COUNT(log.id) FROM $tablename AS log LEFT JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent $where_string)
                    * 100 AS hit_percentage,
                    MAX(datetime) AS last_page_view
                FROM $tablename AS log
                INNER JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent
                $where_string $group_by_string $order_by_string $limit_string $offset_string";
                
        $stmt = self::$__CONN__->prepare($sql);
        if (!$stmt->execute())
        {
            return false;
        }

        // Run!
        if ($limit == 1)
        {
            $object = $stmt->fetchObject(__CLASS__);
            return $object;
        }
        else
        {
            $objects = array();
            while ($object = $stmt->fetchObject(__CLASS__))
            {
                $objects[] = $object;
            }
            return $objects;
        }
    }
    
    public static function findBrowserFamily($limit = 10) {
        return self::find(array(
            'group' => 'ua.ua_family',
            'order' => 'visitor_count DESC, hit_count DESC',
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator' AND ua.ua_family <> 'unknown'",
            'limit' => $limit
        ));
    }
    
    public static function findBrowser($limit = 10) {
        return self::find(array(
            'group' => 'ua.ua_name',
            'order' => 'visitor_count DESC, hit_count DESC',
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator' AND ua.ua_name <> 'unknown'",
            'limit' => $limit
        ));
    }
    
    public static function findOSFamily($limit = 10) {
        return self::find(array(
            'group' => 'ua.os_family',
            'order' => 'visitor_count DESC, hit_count DESC',
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator' AND ua.os_family <> 'unknown'",
            'limit' => $limit
        ));
    }
    
    public static function findOS($limit = 10) {
        return self::find(array(
            'group' => 'ua.os_name',
            'order' => 'visitor_count DESC, hit_count DESC',
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator' AND ua.os_name <> 'unknown'",
            'limit' => $limit
        ));
    }
    
    public static function findRobot($limit = 10) {
        return self::find(array(
            'group' => 'ua.ua_family',
            'order' => 'hit_count DESC',
            'where' => "ua.type = 'Robot' || ua.type = 'Validator'",
            'limit' => (int) $limit
        ));
    }
}