<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /models/VisitorsPage.php
* @date         20/12/2011
*/

class VisitorsPage extends Record
{
    
    public static function find($args = null) {
        // Collect attributes...
        $where    = isset($args['where']) ? trim($args['where']) : '';
        $order_by = isset($args['order']) ? trim($args['order']) : '';
        $offset   = isset($args['offset']) ? (int) $args['offset'] : 0;
        $limit    = isset($args['limit']) ? (int) $args['limit'] : 0;

        // Prepare query parts
        $where_string = empty($where) ? '' : "WHERE $where";
        $order_by_string = empty($order_by) ? '' : "ORDER BY $order_by";
        $limit_string = $limit > 0 ? "LIMIT $limit" : '';
        $offset_string = $offset > 0 ? "OFFSET $offset" : '';

        $tablename = self::tableNameFromClassName('VisitorsRequest');
        $user_agent_table = self::tableNameFromClassName('VisitorsUserAgent');

        // Prepare SQL
        $sql = "SELECT
                    uri,
                    COUNT(uri) AS hits,
                    (COUNT(uri)/(SELECT COUNT(uri) FROM $tablename AS log LEFT JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent WHERE ua.type <> 'Robot') * 100) AS hit_percentage,
                    COUNT(DISTINCT remote_addr, http_referrer) AS visitors,
                    (COUNT(DISTINCT remote_addr, http_referrer)/(SELECT COUNT(DISTINCT remote_addr, http_referrer) FROM $tablename AS log) * 100) AS visitor_percentage
                FROM $tablename AS log
                LEFT JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent
                $where_string
                GROUP BY uri
                $order_by_string $limit_string $offset_string";
                
        $stmt = self::$__CONN__->prepare($sql);
        if (!$stmt->execute())
        {
            return false;
        }

        // Run!
        if ($limit == 1)
        {
            $object = $stmt->fetchObject('VisitorsPage');
            return $object;
        }
        else
        {
            $objects = array();
            while ($object = $stmt->fetchObject('VisitorsPage'))
            {
                $objects[] = $object;
            }
            return $objects;
        }
    }
    
    public static function findTop($limit = 0) {
        return self::find(array(
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator'",
            'order' => 'hits DESC',
            'limit' => $limit
        ));
    }
}