<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /models/VisitorsRequest.php
* @date         20/12/2011
*/

use_helper('Validate');

class VisitorsRequest extends Record
{
    const TABLE_NAME = 'visitors_request';
    
    public function beforeInsert() {
        $this->datetime = date('Y-m-d H:i:s');
        
        if (isset($_SERVER['REMOTE_ADDR'])) {
            if (Validate::ip($_SERVER['REMOTE_ADDR'])) {
                $this->remote_addr = $_SERVER['REMOTE_ADDR'];
                $this->remote_host = gethostbyaddr($this->remote_addr);
                
                $country = $this->parseCountry($this->remote_addr);
                $this->country_icon = $country['icon'];
                $this->country_name = $country['name'];
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
        
        if (isset($_SERVER['HTTP_REFERER']) && Validate::url($_SERVER['HTTP_REFERER'])) {
            $this->http_referrer = $_SERVER['HTTP_REFERER'];
            $referrer = $this->parseReferrer($_SERVER['HTTP_REFERER']);
            $this->referrer_domain = $referrer['domain'];
            $this->referrer_keywords = strtolower($referrer['keywords']);
        }
        
        if (isset($_SERVER['HTTP_USER_AGENT'])) {
            $this->http_user_agent = $_SERVER['HTTP_USER_AGENT'];
        }
        
        if (!$user_agent = VisitorsUserAgent::findByUserAgentString($this->http_user_agent)) {
            $user_agent = new VisitorsUserAgent();
            $user_agent->user_agent_string = $this->http_user_agent;
            $user_agent->save();
        }
        
        $this->session_id = session_id();
        
        if (AuthUser::isLoggedIn()) {
            $this->user_id = (int) AuthUser::getRecord()->id;
        }
        else {
            $this->user_id = NULL;
        }
        
        return true;
    }
    
    public static function parseCountry($ip) {
        $long_ip = sprintf("%u", ip2long($ip));
        
        $ret = self::countryParseHelper(VISITORS."/data/ip-to-country-1.csv", $long_ip);
        
        if ($ret === false) {
            $ret = self::countryParseHelper(VISITORS."/data/ip-to-country-2.csv", $long_ip);
        }
        
        if($ret === false){
            if(!isset($ret['icon'])){
                $ret['icon'] = "unknown.png";
            }
            if(!isset($ret['name'])){
                $ret['name'] = "Unknown";
            }
        }
        
        return $ret;
    }
    
    public static function countryParseHelper($filename, $lookupip_long){
    if(is_readable($filename)){
        $handle = fopen($filename,"r");
            
        if($handle){
            //Seek to the end
            fseek($handle, 0, SEEK_END);
            $high = ftell($handle);
            $low = 0;
            
            while ($low <= $high) {
                $mid = floor(($low + $high) / 2);
        
                fseek($handle, $mid);
    
                if($mid != 0){
                    //Read a line to move to eol
                    $line = fgets($handle);
                }
        
                //Read a line to get data
                $data = fgetcsv ($handle, 200, ",");
                
                if(($data[0] <= $lookupip_long) and ($lookupip_long <= $data[1])){
                    $returnar['icon'] = strtolower($data[2]).".png";
                    $returnar['name'] = ucwords(strtolower($data[4]));
                    fclose($handle);
                    return $returnar;
                }else{
                    if ($lookupip_long < $data[0]){
                        $high = $mid - 1;
                    }else{
                        $low = $mid + 1;
                    }
                }
            }
        }
        fclose($handle);
    }
    return false;
}
    
    public static function parseReferrer($referrer) {
        if (isset($referrer) && $referrer != '') {
        
            $preg = "#(?:\?|&)(?:query|text|words|eingabe|q|p|r)=.+?(?=&)#is";
            preg_match($preg, $referrer, $text);
            
            if(isset($text[0])){
                $keywords = $text[0];
                $keywords = strstr($keywords, "=");
                $keywords = trim($keywords, "=&");
                $keywords = htmlspecialchars(urldecode($keywords));
                
                $ret["keywords"] = $keywords;
            }
            else {
                $ret["keywords"] = "";
            }
            
            $preg = "#http(?:s)?://.+?/#i";
            preg_match($preg, $referrer, $text);
            
            if(isset($text[0])){
                $domain = $text[0];
                $domain = strstr($domain, "/");
                $domain = trim($domain, "/");
                $domain = htmlspecialchars(urldecode($domain));
                $ret["domain"] = $domain;
            }
            else {
                $ret["domain"] = "";
            }
            
            return $ret;
        }
    }
}