<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /models/VisitorsUserAgent.php
* @date         27/12/2011
*/

class VisitorsUserAgent extends Record
{
    const TABLE_NAME = 'visitors_user_agent';
    
    public function beforeInsert() {
        $parser = new UASparser();
        $parser->SetCacheDir(VISITORS."/data/");
        $parsed = $parser->Parse($this->user_agent_string);
        
        $this->type = $parsed['typ'];
        $this->ua_family = $parsed['ua_family'];
        $this->ua_name = $parsed['ua_name'];
        $this->ua_url = $parsed['ua_url'];
        $this->ua_company = $parsed['ua_company'];
        $this->ua_company_url = $parsed['ua_company_url'];
        $this->ua_icon = $parsed['ua_icon'];
        $this->ua_info_url = $parsed['ua_info_url'];
        $this->os_family = $parsed['os_family'];
        $this->os_name = $parsed['os_name'];
        $this->os_url = $parsed['os_url'];
        $this->os_company = $parsed['os_company'];
        $this->os_company_url = $parsed['os_company_url'];
        $this->os_icon = $parsed['os_icon'];
        
        return true;
    }
    
    public static function find($args = null) {
        // Collect attributes...
        $where    = isset($args['where']) ? trim($args['where']) : '';
        $group_by = isset($args['group']) ? trim($args['group']) : '';
        $order_by = isset($args['order']) ? trim($args['order']) : '';
        $limit    = isset($args['limit']) ? (int) $args['limit'] : 0;
        $offset   = isset($args['offset']) ? (int) $args['offset'] : 0;

        // Prepare query parts
        $where_string       = empty($where) ? '' : "WHERE $where";
        $group_by_string    = empty($group_by) ? '' : "GROUP BY $group_by";
        $order_by_string    = empty($order_by) ? '' : "ORDER BY $order_by";
        $limit_string       = $limit > 0 ? "LIMIT $limit" : '';
        $offset_string      = $offset > 0 ? "OFFSET $offset" : '';
        
        $tablename = self::tableNameFromClassName(__CLASS__);
        
        // Prepare SQL
        $sql = "SELECT DISTINCT ua.*
                FROM $tablename AS ua
                $where_string $group_by_string $order_by_string $limit_string $offset_string";
                
        $stmt = self::$__CONN__->prepare($sql);
        if (!$stmt->execute())
        {
            return false;
        }

        // Run!
        if ($limit == 1)
        {
            $object = $stmt->fetchObject(__CLASS__);
            return $object;
        }
        else
        {
            $objects = array();
            while ($object = $stmt->fetchObject(__CLASS__))
            {
                $objects[] = $object;
            }
            return $objects;
        }
    }
    
    public static function findByUserAgentString($user_agent_string) {
        return self::find(array(
            'where' => 'user_agent_string = ' . Record::escape($user_agent_string),
            'limit' => 1
        ));
    }
}