<?php
if (!defined('IN_CMS')) { exit(); }

/**
* Visitors
* 
* Plugin for collecting and analysing visitor statistics.
* 
* @author Nic Wortel <nd.wortel@gmail.com>
* 
* @file         /models/VisitorsVisitor.php
* @date         20/12/2011
*/

class VisitorsVisitor extends Record
{
    
    public function __construct() {
    
        if ($this->user_name != '') {
            $this->visitor = '<a href="' . get_url("user/edit/" . $this->user_id). '">' . $this->user_name . '</a>';
        }
        else {
            $this->visitor = $this->remote_host;
            //$this->visitor = '';
        }
        
    }
    
    public static function date($datetime) {
        $today = date('Y-m-d');
        $yesterday = date('Y-m-d', time()-86400);
        $date = substr($datetime,0,10);
        
        if ($date == $today) {
            return date('H:i', strtotime($datetime));
        }
        elseif ($date == $yesterday) {
            return 'Yesterday';
        }
        else {
            return date('d-m-Y', strtotime($datetime));
        } 
    }
    
    public static function find($args = null) {
        // Collect attributes...
        $where    = isset($args['where']) ? trim($args['where']) : '';
        $group_by = isset($args['group']) ? trim($args['group']) : '';
        $having   = isset($args['having']) ? trim($args['having']) : '';
        $order_by = isset($args['order']) ? trim($args['order']) : '';
        $offset   = isset($args['offset']) ? (int) $args['offset'] : 0;
        $limit    = isset($args['limit']) ? (int) $args['limit'] : 0;

        // Prepare query parts
        $where_string = empty($where) ? '' : "WHERE $where";
        $group_by_string = empty($group_by) ? '' : "GROUP BY $group_by";
        $having_string = empty($having) ? '' : "HAVING $having";
        $order_by_string = empty($order_by) ? '' : "ORDER BY $order_by";
        $limit_string = $limit > 0 ? "LIMIT $limit" : '';
        $offset_string = $offset > 0 ? "OFFSET $offset" : '';

        $tablename = self::tableNameFromClassName('VisitorsRequest');
        $user_table = self::tableNameFromClassName('User');
        $user_agent_table = self::tableNameFromClassName('VisitorsUserAgent');

        // Prepare SQL
        $sql = "SELECT log.*, user.name AS user_name, MAX(log.datetime) AS last_page_view, COUNT(log.uri) AS total_page_views, COUNT(DISTINCT session_id) AS visits, ua.*
                FROM $tablename AS log
                LEFT JOIN $user_table AS user ON user.id = log.user_id
                LEFT JOIN $user_agent_table AS ua ON ua.user_agent_string = log.http_user_agent
                $where_string $group_by_string $having_string $order_by_string $limit_string $offset_string";
                
                
        $stmt = self::$__CONN__->prepare($sql);
        if (!$stmt->execute())
        {
            return false;
        }

        // Run!
        if ($limit == 1)
        {
            $object = $stmt->fetchObject('VisitorsVisitor');
            return $object;
        }
        else
        {
            $objects = array();
            while ($object = $stmt->fetchObject('VisitorsVisitor'))
            {
                $objects[] = $object;
            }
            return $objects;
        }
    }
    
    public static function findRecentHuman($limit = 0) {
        return self::find(array(
            'group' => 'log.remote_addr, log.http_user_agent',
            'order' => 'last_page_view DESC',
            'where' => "ua.type <> 'Robot' AND ua.type <> 'Validator'",
            'limit' => $limit
        ));
    }
    
    public static function findRecentRobot($limit = 0) {
        return self::find(array(
            'group' => 'ua.ua_family',
            'order' => 'last_page_view DESC',
            'where' => "ua.type = 'Robot' AND ua.type <> 'Validator'",
            'limit' => $limit
        ));
    }
    
    public function lastPageView() {
        return self::date($this->last_page_view);
    }
}